-------------------------------------------------------------------------------
-- Copyright (c) 2007 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 2.0
--  \   \        Filename: MultiBoot_module.vhd
--  /   /        Date Last Modified:  April 10 2007
-- /___/   /\    Date Created: Jan 6 2006
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Spartan 3E
--Purpose: This is the module that enables the MultiBoot functionality
--         in any FPGA using Platform Flash PROMs for configuration
--Reference:
--    XAPP483 MultipleBoot with Platform Flash PROMs
--Revision History:
--    Rev 1.0 Release version, selectable REV_SEL bus width
--    Rev 2.0 Enhanced version with added safety features
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;


entity multiBoot_module is
	port (	--use the following two lines if switching between 3 or 4 revisons
			   
				REV_sel_OUT: out std_logic_vector(1 downto 0);		--for 3 or 4 revisions only
				REV_sel_IN : in std_logic_vector(1 downto 0);		--for 3 or 4 revisions only
				
				--un-comment the following two lines if switching between only 2 revisions and ground REV_SEL[1] on PROM
				--REV_sel_OUT: out std_logic;				--for 2 revisions only
				--REV_sel_IN : in std_logic;				--for 2 revisions only
	
				sysclk: in  std_logic;
            PROG: out std_logic;
			   CF: out std_logic;
		   	MBT : in std_logic;
				RESET: in std_logic;
				INIT: out std_logic;
				DIN: in std_logic_vector(7 downto 0));
end multiBoot_module;

architecture Behavioral of multiBoot_module is

	signal l_count: integer range 0 to 50000000 := 0;
	signal didNotWork: std_logic:='0';
	signal PROG_control: std_logic:='1';
	signal CF_control: std_logic:='1';
	signal INIT_control: std_logic:='0';
	type state_type is (ST0,ST1,ST2,ST3,ST4,ST5,ST6,ST_initialize);
	signal NS : state_type;

begin  
   
 
		
  process (NS,sysclk)
   begin
    	
		if(sysclk'event and sysclk = '1') then
			if(RESET ='1')then
				NS <= ST_initialize;
			else
			
				case NS is 
				
					when ST_initialize=>
						CF_control <= '1';
						PROG_control <= '1';
						INIT_control <='0';
						l_count<= 0;
						didNotWork<='0';
						NS<=ST0;
	
	
					when ST0 =>	 					-- state 0:  check for trigger
				
						if(didNotWork='0') then
							if(MBT='0'and NS=ST0)	then
								NS<=ST1;
							else
								NS<=ST0;
							end if;
						else
							NS<=ST6;
						end if;				
						
	
					when ST1 =>	 					-- state 1: set CF=LOW and REV_SEL
						if(l_count >= 25000000) then		
							CF_control <= '0';	
							l_count<=0;						
							NS<=ST2;
						else
						   l_count <= l_count + 1;
							NS<=ST1;
						end if;
											
					when ST2=>						--state 2: set CF=HIGH and INIT=HIGH after 300ns or Rev_sel set up time	
						if(l_count >=15) then
							l_count <=0;	
							CF_control <= '1';
							INIT_control <= '1';
							NS<=ST3;
						else
						   l_count <= l_count + 1;
							NS<=ST2;
						end if;
	
			
					
					when ST3 =>						--state 3: check data on D[7:0], for 0x99, if not present then go to idle state
						if(l_count>=15) then		----wait 300ns Rev_sel hold time	before checking data	
							if(DIN="10011001") then		-- wait to make sure data on the bus is valid
								NS<=ST4;
								l_count<=0;
								INIT_control<='0';
								CF_control<='0';
							else
								NS<=ST6;
								l_count <=0;
								didNotWork <= '1';
								INIT_control <= '0';
								CF_control<='1';
							end if;
						else
					  	  l_count <= l_count + 1;
							NS<=ST3;
						end if;
					
					when ST4 =>	  					--state 4: set CF=LOW and rev_sel, setup for reconfig
						if(l_count >= 5) then		
							CF_control <= '0';	
							INIT_control <='0';
							l_count<=0;						
							NS<=ST5;
						else
						   l_count <= l_count + 1;
							NS<=ST4;
						end if;
											
					when ST5=>						--state 5: set CF=HIGH and set PROG=LOW after 300ns or Rev_sel set up time	
						if(l_count >=15) then
							l_count <=0;	
							CF_control <= '1';
							PROG_control <= '0';
							NS<=ST6;
						else
						   l_count <= l_count + 1;
							NS<=ST5;
						end if;						
	
					when ST6 =>					-- do nothing except await a RESET signal
						--wait for RESET signal
				end case;

			end if;
		end if;
     end process ;

    
	  PROG <= PROG_control;
	  CF <= CF_control;
	  INIT<= INIT_control;
	  REV_sel_OUT <= REV_sel_IN;


end Behavioral;
