-------------------------------------------------------------------------------
-- Copyright (c) 2006 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: moduleTest.vhd
--  /   /        Date Last Modified:  Jun 1 2006
-- /___/   /\    Date Created: Jan 6 2006
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Spartan 3E
--Purpose: This is the module that tests out the MultiBoot functionality
--         using Platform Flash PROMs, simulating a real application that 
--			  would trigger MultipleBoot module and select the revision
--Reference:
--    XAPP483 MultipleBoot with Platform Flash PROMs
--Revision History:
--    Rev 1.0 For testing purposed, successfully simulated an application
--		that would use the MultiBoot module, by sending the REV_SEL and MBT trigger
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity moduleTest is
	port (sysclk: in  std_logic;
          led   : out std_logic_vector(7 downto 0)	;
			 MBT_in: in std_logic;
			 MBT_out: out std_logic;
			 REV_SEL_out: out std_logic_vector(1 downto 0);
			 RESET_out: out std_logic);
end moduleTest;

architecture Behavioral of moduleTest is

	signal count: std_logic_vector(7 downto 0):= "11110000";
	signal l_count: integer range 0 to 50000000 := 0;
	signal b_count: integer range 0 to 100000000 := 0;
	signal MBT_control: std_logic:='1';
	signal RESET_control: std_logic:='0';		 
	signal attempting_MultiBoot: std_logic:= '0';
	signal REV_SEL_switch: std_logic_vector(1 downto 0):="00";

begin  
   	process (sysclk, count, l_count,b_count,MBT_in)
    	begin
    	
			if(sysclk'event and sysclk = '1') then
				l_count <= l_count + 1;
				if(l_count >= 50000000) then				
					l_count <= 0;
					count <= not count;					--toggle dummy LED display every second based on 50Mhz clock
				end if;
				
				if(MBT_in = '0'and attempting_MultiBoot = '0') then --check for MBT trigger
					MBT_control <='0';			
					attempting_MultiBoot <= '1';
				end if;
				if (attempting_MultiBoot='1') then		--simulate the pulse of MBT out put
					b_count <= b_count + 1;
					if(b_count=10000000) then
						MBT_control <='1';
					end if;

					if(b_count=75000000) then			--RESET the Multiboot Module after timeout
						RESET_control <='1';
					end if;
					if(b_count>=100000000) then
						b_count <= 0;
						RESET_control <= '0';
						attempting_MultiBoot <='0';
						
					end if;
				end if;

			end if;

     end process;

    
     led(7 downto 0) <=  count(7 downto 0);
	  MBT_out <= MBT_control;
	  RESET_out <= RESET_control;
	  REV_SEL_out<= REV_SEL_switch;


end Behavioral;